# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of dgt calculation

Ported from ltfat_2.1.0/gabor/dgt.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.comp.comp_sepdgt import comp_sepdgt
from ltfatpy.gabor.dgtlength import dgtlength
from ltfatpy.gabor.gabwin import gabwin
from ltfatpy.comp.assert_groworder import assert_groworder
from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post


def dgt(f, g, a, M, L=None, pt='freqinv'):
    """Discrete Gabor Transform

    - Usage:

        | ``(c, Ls, g) = dgt(f, g, a, M)``
        | ``(c, Ls, g) = dgt(f, g, a, M, L)``
        | ``(c, Ls, g) = dgt(f, g, a, M, L, pt)``

    - Input parameters:

    :param numpy.ndarray f: Input data. **f** dtype has to be float64 or
        complex128.
    :param g: Window function.
    :param int a: Length of time shift.
    :param int M: Number of channels.
    :param int L: Length of transform to do. Default is None.
    :param str pt: 'freqinv' or 'timeinv'. Default is 'freqinv'
    :type g: str, dict or numpy.ndarray

    - Output parameters:

    :returns: ``(c, Ls, g)``
    :rtype: tuple
    :var numpy.ndarray c: :math:`M*N` array of gabor transform coefficients.
        Its dtype is complex128.
    :var int Ls: length of input signal
    :var numpy.ndarray g: updated window function. Its dtype is float64 or
        complex128 depending on **f** dtype.

    ``dgt(f, g, a, M)`` computes the Gabor coefficients (also known as a
    windowed Fourier transform) of the input signal **f** with respect to the
    window **g** and parameters **a** and **M**. The output is a one or two
    dimensional :class:`numpy.ndarray` in a rectangular layout.

    The length of the transform will be the smallest multiple of **a** and
    **M** that is larger than the signal. **f** will be zero-extended to the
    length of the transform. If **f** is a 2d array, the transformation is
    applied to each column. The length of the transform done can be obtained
    by ``L = c.shape[1] * a``

    The window **g** may be an array of numerical values, a text string or a
    dictionary. See the help of :func:`~ltfatpy.gabor.gabwin` for more
    details.

    ``dgt(f, g, a, M, L)`` computes the Gabor coefficients as above, but does
    a transform of length **L**. **f** will be cut or zero-extended to length
    **L** before the transform is done.

    ``(c, Ls) = dgt(f, g, a, M)`` or ``(c, Ls) = dgt(f, g, a, M, L)`` returns
    the length of the input signal **f**. This is handy for reconstruction:

    - Examples:

    In the following example we create a Hermite function, which is a
    complex-valued function with a circular spectrogram, and visualize
    the coefficients using both :func:`~matplotlib.pyplot.imshow` and
    :func:`~ltfatpy.gabor.plotdgt.plotdgt`:

    >>> import numpy as np
    >>> import matplotlib.pyplot as plt
    >>> from ltfatpy import plotdgt
    >>> from ltfatpy import pherm
    >>> a = 10
    >>> M = 40
    >>> L = a * M
    >>> h, _ = pherm(L, 4)  # 4th order hermite function.
    >>> c = dgt(h, 'gauss', a, M)[0]
    >>> # Simple plot: The squared modulus of the coefficients on
    >>> # a linear scale
    >>> _ = plt.imshow(np.abs(c)**2, interpolation='nearest', origin='upper')
    >>> plt.show()
    >>> # Better plot: zero-frequency is displayed in the middle,
    >>> # and the coefficients are show on a logarithmic scale.
    >>> _ = plotdgt(c, a, dynrange=50)
    >>> plt.show()

    .. image:: images/dgt_1.png
       :width: 700px
       :alt: imshow image
       :align: center
    .. image:: images/dgt_2.png
       :width: 600px
       :alt: plotdgt image
       :align: center

    ``(c, Ls, g)=dgt(...)`` outputs the window used in the transform. This is
    useful if the window was generated from a description in a string or
    dictionary.

    The Discrete Gabor Transform is defined as follows: Consider a window
    **g** and a one-dimensional signal **f** of length **L** and define
    :math:`N = L / a`. The output from ``c = dgt(f, g, a, M)`` is then given
    by:

    .. math::

        c\\left(m+1,n+1\\right)=\\sum_{l=0}^{L-1}f(l+1)\\overline{g(l-an+1)}
        e^{-2\\pi ilm/M}

    where :math:`m=0,\\ldots,M-1`, :math:`n=0, \\ldots,N-1` and :math:`l-an`
    are computed modulo **L**.

    - Additional parameters:

        ``dgt`` takes the following keyword at the end of the line of input
        arguments:

        pt = 'freqinv'
            Compute a DGT using a frequency-invariant phase. This
            is the default convention described above.

        pt = 'timeinv'
            Compute a DGT using a time-invariant phase. This
            convention is typically used in FIR-filter algorithms.

    .. seealso::  :func:`~ltfatpy.gabor.idgt.idgt`,
        :func:`~ltfatpy.gabor.gabwin.gabwin`, :func:`dwilt`,
        :func:`~ltfatpy.gabor.gabdual.gabdual`,
        :func:`~ltfatpy.gabor.phaselock.phaselock`

    - References:
        :cite:`fest98,gr01`
    """

    # Verify f and determine its length
    # Change f to correct shape.
    (f, _unused, Ls, W, dim, permutedshape, order) = assert_sigreshape_pre(f,
                                                                           L)

    # Verify a, M and L
    if L is None:
        # Verify a, M and get L from the signal length f
        L = dgtlength(Ls, a, M)
    else:
        # Verify a, M and get L
        Luser = dgtlength(L, a, M)
        if Luser != L:
            raise ValueError(("Incorrect transform length L={0:d} specified." +
                              " Next valid length  is L={1:d}. See the help" +
                              " of DGTLENGTH for the requirements.").
                             format(L, Luser))
    # verify pt
    if pt == 'timeinv':
        pt = 1
    elif pt == 'freqinv':
        pt = 0
    else:
        raise ValueError("pt argument should be 'timeinv' or 'freqinv'.")
    # Determine the window
    (gnum, info) = gabwin(g, a, M, L)
    if L < info['gl']:
        raise ValueError('Window is too long.')

    # final cleanup
    # Postpad
    C = 0
    if Ls < L:
        f = np.concatenate((f, C*np.ones((L-Ls, W))), axis=0)
    else:
        f = f[:L]

    # call the computation subroutines
    c = comp_sepdgt(f, gnum, a, M, pt)
    # flags_do_timeinv = 1
    order = assert_groworder(order)
    permutedshape = (M, L//a) + permutedshape[1:]
    c = assert_sigreshape_post(c, dim, permutedshape, order)

    if [i for i in c.shape if i > 2] and c.shape[0] == 1:
        c = c.squeeze()

    return (c, Ls, gnum)

if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of frequency index of FFT modulations computation

Ported from ltfat_2.1.0/fourier/fftindex.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np


def fftindex(N, nyquistzero=False):
    """Frequency index of FFT modulations

    - Usage:

        | ``n = fftindex(N)``

    - Input parameters:

    :param int N: FFT length
    :param bool nyquistzero: If ``True``, sets the Nyquist frequency to zero

    - Output parameters:

    :returns: Indexes of the frequencies of the standard FFT
    :rtype: numpy.ndarray

    ``fftindex(N)`` returns the index of the frequencies of the standard FFT
    of length **N** as they are ordered in the output from the
    :func:`numpy.fft.fft` routine. The numbers returned are in the range
    ``-ceil(N/2)+1:floor(N/2)``.

    ``fftindex(N, True)`` does as above, but sets the Nyquist frequency to
    zero.

    .. seealso:: :func:`dft`
    """

    if not nyquistzero:
        if N % 2 == 0:
            n = np.r_[0:N//2+1, -N//2+1:0]
        else:
            n = np.r_[0:(N+1)//2, -(N-1)//2:0]
    else:
        if N % 2 == 0:
            n = np.r_[0:N//2, 0, -N//2+1:0]
        else:
            n = np.r_[0:(N+1)//2, -(N-1)//2:0]

    return n



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of dimension input proprocessing and handling

Ported from ltfat_2.1.0/comp/assert_sigreshape_pre.m

.. moduleauthor:: Denis Arrivault
                  Florent Jaillet
"""

import numpy as np
import sys


def assert_sigreshape_pre(f, L=None, dim=None):
    """Preprocess and handle dimension input

    - Input parameters:

    :param numpy.ndarray f: Signal
    :param int L: L parameter
    :param int dim: dim parameter

    - Output parameters:

    :returns: ``(f, L, Ls, W, dim, permutedshape, order)``
    :rtype: tuple

    :var numpy.ndarray f: Input signal as matrix.
    :var int L: Verified L.
    :var int Ls: Length of signal along dimension to be processed.
    :var W: Number of transforms to do.
    :vartype W: int
    :var int dim: Verified dim.
    :var tuple permutedshape: Pass to
        :func:`~ltfatpy.comp.assert_sigreshape_post.assert_sigreshape_post`
    :var tuple order: Pass to
        :func:`~ltfatpy.comp.assert_sigreshape_post.assert_sigreshape_post`

    .. warning::
        This function returns a view of **f**. Any value changed in this
        view will also be changed in **f**.
    """

# ---- Format f as a numpy array
    if type(f).__module__ != np.__name__:
        f = np.asarray(f)

# ----Check that f type is numeric
    if (not np.issubdtype(f.dtype, np.floating) and
       not np.issubdtype(f.dtype, np.complexfloating) and
       not np.issubdtype(f.dtype, np.integer)):
        raise TypeError("f data type should be numeric.")
    D = f.ndim
    order = (1,)
    if dim is None:
        dim = 0
        if (sum(item > 1 for item in f.shape) == 1):
            # ---- We have a vector, find the dimension where it lives.
            dim = [i for i, v in enumerate(f.shape) if v > 1][0]
    else:
        if (not isinstance(dim, int) and
           not isinstance(dim, np.int_)):
            raise TypeError("dim should be an integer.")
        if dim < 0 or dim > D:
            raise TypeError("dim must be in the range from 0 to "
                            "{0:d}.".format(D-1))

    if L is not None:
        if not isinstance(L, int):
            raise TypeError("L should be an integer.")

    if dim > 0:
        # NOTE: the use of list below is needed for Python3 compatibility
        order = tuple([dim]+list(range(0, dim))+list(range(dim+1, D)))
        # ---- Put the desired dimension first.
        f = np.transpose(f, order)

    Ls = f.shape[0]
    # ---- If L is None it is set to be the length of the transform.
    if L is None:
        L = Ls

    # ---- Remember the exact shape for later and modify it for the new length
    permutedshape = f.shape
    permutedshape = (L,) + permutedshape[1:]

    # ---- Reshape f to a matrix.
    if f.size != 0:
        f = np.reshape(f, (f.shape[0], f.size // f.shape[0]))

    W = f.shape[1]

    return (f, L, Ls, W, dim, permutedshape, order)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of instantaneous frequency plot

Ported from ltfat_2.1.0/gabor/instfreqplot.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.gabor.plotdgt import plotdgt
from ltfatpy.gabor.plotdgtreal import plotdgtreal
from ltfatpy.gabor.gabimagepars import gabimagepars
from ltfatpy.fourier.fftresample import fftresample
from ltfatpy.gabor.dgt import dgt
from ltfatpy.gabor.gabphasegrad import gabphasegrad


def instfreqplot(f, fs=None, tfr=1., wlen=None, nf=None, thr=None, fmax=None,
                 xres=800, yres=600, method='dgt', climsym=None, **kwargs):
    """Plot of instantaneous frequency

    - Input parameters:

    :param numpy.ndarray f: Analyzed signal
    :param float fs: Sampling rate in Hz of the analyzed signal
    :param float tfr: Set the ratio of frequency resolution to time resolution.
        A value ``tfr = 1.0`` is the default. Setting ``tfr > 1.0`` will give
        better frequency resolution at the expense of a worse time resolution.
        A value of ``0.0 < tfr < 1.0`` will do the opposite.
    :param int wlen: Window length. Specifies the length of the window
        measured in samples. See help of :func:`~ltfatpy.fourier.pgauss` on the
        exact details of the window length (parameter width).
    :param bool nf: If ``True``, display negative frequencies, with the
        zero-frequency centered in the middle. For real signals, this will just
        mirror the upper half plane. This is standard for complex signals.
    :param float thr: Keep the coefficients with a magnitude larger than
        **thr** times the largest magnitude. Set the instantaneous frequency of
        the rest of the coefficients to zero.
    :param float fmax: Display **fmax** as the highest frequency.
    :param int xres: Approximate number of pixels in the time direction
    :param int yres: Number of pixels in the frequency direction
    :param str method: Specify the method used for instantaneous frequency
        computation. Possible values are 'dgt', 'phase' and 'abs'. See the help
        of :func:`~ltfatpy.gabor.gabphasegrad` for details.
    :param float climsym: Use a colormap ranging from ``-climsym`` to
        ``+climsym``
    :param `**kwargs`: ``instfreqplot`` supports all the optional parameters of
        :func:`~ltfatpy.gabor.tfplot.tfplot`. Please see the help of
        :func:`~ltfatpy.gabor.tfplot.tfplot` for an exhaustive list.

    - Output parameters:

    :returns: The instantaneous frequency data used in the plotting
    :rtype: numpy.ndarray

    ``instfreqplot(f)`` plots the instantaneous frequency of **f** using a
    Discrete Gabor Transform (:func:`~ltfatpy.gabor.dgt.dgt`).

    The instantaneous frequency contains extreme spikes in regions
    where the spectrogram is close to zero. These points are usually
    uninteresting and destroy the visibility of the plot. Use the thr
    or clim or climsym options to remove these points.

    - An example:

        >>> from ltfatpy import instfreqplot, greasy
        >>> from matplotlib.pyplot import show
        >>> _ = instfreqplot(greasy()[0], 16000., thr=.03, climsym=100.)
        >>> show()

        will produce a nice instantaneous frequency plot of the
        :func:`~ltfatpy.signals.greasy.greasy` signal.

    .. image:: images/instfreqplot.png
       :width: 600px
       :alt: instantaneous frequency of the greasy signal
       :align: center

    .. seealso:: :func:`~ltfatpy.gabor.sgram.sgram`, :func:`resgram`,
        :func:`~ltfatpy.gabor.phaseplot.phaseplot`
    """

    if not isinstance(f, np.ndarray):
        raise TypeError('f must be a 1D numpy.ndarray')

    if f.ndim > 1:
        raise ValueError('Input must be a vector.')

    if nf is None:
        if np.isrealobj(f):
            nf = False
        else:
            nf = True

    # Override the setting from tfplot, because instfreqplot does not support
    # the dB-settings
    kwargs['normalization'] = 'lin'

    # Downsample
    if fmax:
        if fs:
            resamp = fmax*2./fs
        else:
            resamp = fmax*2./f.shape[0]

        f = fftresample(f, int(round(f.shape[0] * resamp)))
        fs = 2. * fmax

    Ls = f.shape[0]

    if not nf:
        yres = 2*yres

    try:
        a, M, L, N, Ndisp = gabimagepars(Ls, xres, yres)

    except ValueError:
        raise ValueError(
            'The signal is too long. instfreqplot cannot visualize all the '
            'details.\nTry a shorter signal or increase the image resolution '
            'by calling:\n\ninstfreqplot(..., xres=xres, yres=yres)\n\n'
            'for larger values of xres and yres.\nThe current values are:\n  '
            'xres = {0}\n  yres = {1}'.format(xres, yres))
    except:
        raise

    # Set an explicit window length, if this was specified.
    if wlen:
        tfr = wlen**2 / L

    g = {'name': 'gauss', 'tfr': tfr}

    if method == 'dgt':
        coef, fgrad, dgtcoef = gabphasegrad('dgt', f, g, a, M)
    elif method == 'phase':
        dgtcoef = dgt(f, g, a, M)[0]
        coef, fgrad = gabphasegrad('phase', np.angle(dgtcoef), a)
    elif method == 'abs':
        dgtcoef = dgt(f, g, a, M)[0]
        coef, fgrad = gabphasegrad('abs', np.abs(dgtcoef), g, a)

    if thr:
        # keep only the largest coefficients.
        maxc = np.max(np.abs(dgtcoef))
        mask = np.abs(dgtcoef) < maxc*thr
        coef[mask] = 0.

    # Cut away zero-extension.
    coef = coef[:, 0:Ndisp]

    # Convert climsym into a normal clim
    if climsym:
        kwargs['clim'] = [-climsym, climsym]
    else:
        # handle default value for clim
        if 'clim' not in kwargs:
            kwargs['clim'] = (0., 1.)

    if nf:
        plotdgt(coef, a, fs=fs, **kwargs)
    else:
        coef = coef[:int(np.floor(M/2))+1, :]
        plotdgtreal(coef, a, M, fs=fs, **kwargs)

    return coef


if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of FIR window names

Ported from ltfat_2.1.0/comp/arg_firwin.m

This module defines the arg_firwin() function

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division


def arg_firwin():
    """Returns the names of FIR windows

    - Output parameters:

    :returns: the names of FIR windows
    :rtype: set containing all the names
    """
    return {'hanning', 'hann', 'sine', 'cosine', 'sqrthann', 'hamming',
            'square', 'rect', 'tria', 'bartlett', 'triangular', 'sqrttria',
            'blackman', 'blackman2', 'nuttall', 'nuttall10', 'nuttall01',
            'nuttall20', 'nuttall11', 'nuttall03', 'nuttall12', 'nuttall21',
            'nuttall30', 'ogg', 'itersine'
            }



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of time-frequency plotting

Ported from ltfat_2.1.0/gabor/tfplot.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np
import matplotlib.pyplot as plt
from matplotlib import cm


def tfplot(coef, step, yr, fs=None, dynrange=None, normalization='db',
           tc=False, clim=None, plottype='image', colorbar=True, display=True,
           time='Time', frequency='Frequency', samples='samples',
           normalized='normalized'):
    """Plot coefficient matrix on the TF plane

    - Input parameters:

    :param numpy.ndarray coef: 2D coefficient array
    :param float step: Shift in samples between each column of coefficients
    :param numpy.ndarray yr: 2 elements vector containing the lowest and
        highest normalized frequency
    :param float fs: Sampling rate in Hz of the original signal
    :param float dynrange: Limit the dynamical range to dynrange by using a
        colormap in the interval [chigh-dynrange, chigh], where chigh is the
        highest value in the plot. The default value of None means to not limit
        the dynamical range. If both **clim** and **dynrange** are specified,
        then **clim** takes precedence.
    :param str normalization: String specifying the normalization of the plot,
        possible values are listed below
    :param bool tc: Time centering: if ``True``, move the beginning of the
        signal to the middle of the plot. This is usefull for visualizing the
        window functions of the toolbox.
    :param tuple clim: Use a colormap ranging from clim[0] to clim[1]. If both
        **clim** and **dynrange** are specified, then **clim** takes
        precedence.
    :param str plottype: String specifying the type of plot, possible values
        are listed below
    :param bool colorbar: If ``True``, display the colorbar (this is the
        default)
    :param bool display: If ``True``, display the figure (this is the default).
        Using ``display=False`` to avoid displaying the figure is usefull if
        you only want to obtain the output for further processing.
    :param str time: Text customization: the word denoting time
    :param str frequency: Text customization: the word denoting frequency
    :param str samples: Text customization: the word denoting samples
    :param str normalized: Text customization: the word denoting normalized

    - Output parameter:

    :returns: The processed image data used in the plotting. Inputting this
              data directly to :func:`~matplotlib.pyplot.matshow` or similar
              functions will create the plot. This is usefull for custom
              post-processing of the image data.
    :rtype: numpy.ndarray

    ``tfplot(coef, step, yr)`` will plot a rectangular coefficient array on the
    TF-plane.

    ``tfplot`` is not meant to be called directly. Instead, it is called by
    other plotting routines to give a uniform display format.


    Possible values for **normalization**:
        ============ ==========================================================
        ``'db'``     Apply :math:`20*\\log_{10}` to the coefficients. This makes
                     it possible to see very weak phenomena, but it might show
                     too much noise. A logarithmic scale is more adapted to
                     perception of sound. This is the default.

        ``'dbsq'``   Apply :math:`10*\\log_{10}` to the coefficients. Same as
                     the ``'db'`` option, but assume that the input is already
                     squared.

        ``'lin'``    Show the coefficients on a linear scale. This will display
                     the raw input without any modifications. Only works for
                     real-valued input.

        ``'linsq'``  Show the square of the coefficients on a linear scale.

        ``'linabs'`` Show the absolute value of the coefficients on a linear
                     scale.
        ============ ==========================================================

    Possible values for **plottype**:
        ============= ====================================================
        ``'image'``   Use imshow to display the plot. This is the default.

        ``'contour'`` Do a contour plot.

        ``'surf'``    Do a surface plot.

        ``'pcolor'``  Do a pcolor plot.
        ============= ====================================================

    .. seealso:: :func:`~ltfatpy.gabor.sgram.sgram`,
        :func:`~ltfatpy.gabor.plotdgt.plotdgt`,
        :func:`~ltfatpy.gabor.plotdgtreal.plotdgtreal`,
        :func:`plotwmdct`, :func:`plotdwilt`

    """

    if not isinstance(coef, np.ndarray):
        raise TypeError('coef must be a 2D numpy.ndarray')

    if coef.ndim > 2:
        raise ValueError('Input is multidimensional. coef must be a 2D '
                         'numpy.ndarray')

    if coef.ndim < 2:
        raise ValueError('coef must be a 2D numpy.ndarray')

    M = coef.shape[0]
    N = coef.shape[1]

    # Apply transformation to coefficients.
    if normalization == 'db':
        coef = 20. * np.log10(np.abs(coef) + np.finfo(np.float64).tiny)
    elif normalization == 'dbsq':
        coef = 10. * np.log10(np.abs(coef) + np.finfo(np.float64).tiny)
    elif normalization == 'linsq':
        coef = np.square(np.abs(coef))
    elif normalization == 'linabs':
        coef = np.abs(coef)
    elif normalization == 'lin':
        if not np.isrealobj(coef):
            raise ValueError("Complex valued input cannot be plotted using the"
                             " 'lin' flag. Please use the 'linsq' or 'linabs' "
                             "flag.")
        else:
            # coef is returned in the output so we make a copy to avoid
            # returning a reference to the data passed in input
            coef = coef.copy()

    # 'dynrange' parameter is handled by converting it into clim
    #  clim overrides dynrange, so do nothing if clim is already specified
    if dynrange and not clim:
        maxclim = np.nanmax(coef)
        clim = (maxclim - dynrange, maxclim)

    # Handle clim by thresholding and cutting
    if clim:
        np.clip(coef, clim[0], clim[1], out=coef)

    if tc:
        xr = np.arange(-np.floor(N/2.), np.floor((N-1)/2)+1) * step
        coef = np.fft.fftshift(coef, axes=1)
    else:
        xr = np.arange(0, N) * step

    if display:
        if fs:
            xr = xr / fs
            yr = yr * fs/2

        # Convert yr to range of values
        yr = np.linspace(yr[0], yr[1], M)

        if plottype == 'image':
            xstep = xr[1] - xr[0]
            ystep = yr[1] - yr[0]
            extent = [xr[0] - xstep/2, xr[-1] + xstep/2,
                      yr[0] - ystep/2,  yr[-1] + ystep/2]

            if clim:
                # Call imshow explicitly with clim. This is necessary for the
                # situations where the data is by itself limited (from above
                # or below) to within the specified range. Setting clim
                # explicitly avoids the colormap moves in the top or bottom.
                plt.imshow(coef, extent=extent, aspect='auto',
                           interpolation='nearest', origin='lower', clim=clim)
            else:
                plt.imshow(coef, extent=extent, aspect='auto',
                           interpolation='nearest', origin='lower')

        elif plottype == 'contour':
            # Note: The matlplotlib contour function doesn't give the exact
            # same visual results as in Octave for the same data.
            # So we can expect some slight differences when comparing contour
            # plots produced by tfplot.
            plt.contour(xr, yr, coef, 10)

        elif plottype == 'surf':
            # Note: The following import is needed to be able to use
            # plot_surface
            from mpl_toolkits.mplot3d import Axes3D

            plt.delaxes()
            ax = plt.gcf().add_subplot(111, projection='3d')
            ax.azim = -130.
            ax.elev = 30.
            xgrid, ygrid = np.meshgrid(xr, yr)
            ax.plot_surface(xgrid, ygrid, coef, rstride=1, cstride=1,
                            antialiased=True, linewidth=0, cmap=cm.jet)
            # Note: matplotlib doesn't support orthogonal projection,
            # so the result doesn't look exactly as in Octave. See:
            # http://stackoverflow.com/questions/23840756/ \
            #                          how-to-disable-perspective-in-mplot3d

        elif plottype == 'pcolor':
            plt.pcolor(xr, yr, coef, edgecolors='k', antialiased=False,
                       linewidth=1)
            plt.axis('tight')

        if colorbar:
            if plottype == 'surf':
                # Note: we can use "plottype in ('contour', 'surf')" in the
                # previous test if we want the colorbar in contour plots to
                # look more like the one in Octave
                mappable = cm.ScalarMappable(cmap=cm.jet)
                mappable.set_array(coef)
                plt.colorbar(mappable, ax=plt.gca())
            else:
                plt.colorbar(ax=plt.gca())

        if fs:
            plt.xlabel(time + ' (s)')
            plt.ylabel(frequency + ' (Hz)')
        else:
            plt.xlabel(time + ' (' + samples + ')')
            plt.ylabel(frequency + ' (' + normalized + ')')

    return coef



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the comp_hermite_all function

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import unittest
import numpy as np
from ltfatpy.comp.comp_hermite_all import comp_hermite_all
# import matplotlib.pyplot as plt


class TestCompHermiteAll(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        self.x = np.arange(10)

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_default(self):
        y = comp_hermite_all(0, self.x)
        yres = np.array([7.51125544e-01, 4.55580672e-01, 1.01653788e-01,
                         8.34425107e-03, 2.51974549e-04, 2.79918439e-06,
                         1.14396268e-08, 1.71987833e-11, 9.51237824e-15,
                         1.93546809e-18])
        np.testing.assert_array_almost_equal(y, yres, 6)

        y = comp_hermite_all(1, self.x)
        yres = np.array([[7.51125544e-01],
                         [4.55580672e-01],
                         [1.01653788e-01],
                         [8.34425107e-03],
                         [2.51974549e-04],
                         [2.79918439e-06],
                         [1.14396268e-08],
                         [1.71987833e-11],
                         [9.51237824e-15],
                         [1.93546809e-18]])
        np.testing.assert_array_almost_equal(y, yres, 6)

        y = comp_hermite_all(2, self.x)
        yres = np.array([[7.51125544e-01, 0.00000000e+00],
                         [4.55580672e-01, 6.44288365e-01],
                         [1.01653788e-01, 2.87520332e-01],
                         [8.34425107e-03, 3.54016591e-02],
                         [2.51974549e-04, 1.42538330e-03],
                         [2.79918439e-06, 1.97932227e-05],
                         [1.14396268e-08, 9.70684525e-08],
                         [1.71987833e-11, 1.70259268e-10],
                         [9.51237824e-15, 1.07620275e-13],
                         [1.93546809e-18, 2.46344870e-17]])
        np.testing.assert_array_almost_equal(y, yres, 6)

        y = comp_hermite_all(3, self.x)
        yres = np.array([[7.51125544e-01, 0.00000000e+00, -5.31125966e-01],
                         [4.55580672e-01, 6.44288365e-01, 3.22144183e-01],
                         [1.01653788e-01, 2.87520332e-01, 5.03160581e-01],
                         [8.34425107e-03, 3.54016591e-02, 1.00304701e-01],
                         [2.51974549e-04, 1.42538330e-03, 5.52336028e-03],
                         [2.79918439e-06, 1.97932227e-05, 9.69867910e-05],
                         [1.14396268e-08, 9.70684525e-08, 5.74321677e-07],
                         [1.71987833e-11, 1.70259268e-10, 1.17965350e-09],
                         [9.51237824e-15, 1.07620275e-13, 8.54235929e-13],
                         [1.93546809e-18, 2.46344870e-17, 2.20341800e-16]])
        np.testing.assert_array_almost_equal(y, yres, 6)

        self.assertRaises(TypeError, comp_hermite_all, 3, "foo")
        self.assertRaises(TypeError, comp_hermite_all, 3, np.ones((1, 1)))

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestCompHermiteAll)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of comp_sigreshape_pre calculation

Ported from ltfat_2.1.0/comp/comp_sigreshape_pre.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np


def comp_sigreshape_pre(f, do_ndim):
    """Set good dimensionality

    .. warning::
        This function returns **f** or a view of **f**. Any value changed in
        the returned variable will also be changed in **f**.
    """
    if not isinstance(f, np.ndarray):
        raise ValueError("f should be a numpy array.")
    elif f.size == 0 or (not np.issubdtype(f.dtype, np.floating) and
                         not np.issubdtype(f.dtype, np.complexfloating) and
                         not np.issubdtype(f.dtype, np.integer)):
        raise ValueError('The input must be non-empty numeric.')

    wasrow = False

    # Rember the shape if f is multidimensional.
    remembershape = f.shape
    fd = len(remembershape)

    # Multi-dimensional mode, apply to first dimension.
    if fd > 2:
        if do_ndim > 0 and fd > do_ndim:
            raise ValueError('Cannot process multidimensional arrays.')

        fl = f.shape[0]
        W = np.prod(remembershape)//fl

        # Reshape to matrix if multidimensional.
        f = f.reshape((fl, W))
    else:
        if f.shape[0] == 1:
            wasrow = True
            # Make f a column vector.
            f = f.squeeze()
        fl = f.shape[0]
        if f.ndim > 1:
            W = f.shape[1]
        else:
            W = 1

    return (f, fl, W, wasrow, remembershape)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of signal resampling using Fourier interpolation

Ported from ltfat_2.1.0/fourier/fftresample.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post
from ltfatpy.tools.postpad import postpad
from ltfatpy.fourier.middlepad import middlepad
from ltfatpy.fourier.fftreal import fftreal
from ltfatpy.fourier.ifftreal import ifftreal


def fftresample(f, L, dim=None):
    """Resample signal using Fourier interpolation

    - Usage:
        | ``h = fftresample(f, L)``
        | ``h = fftresample(f, L, dim)``

    - Input parameters:

    :param numpy.ndarray f: Input array
    :param int L: Length of the output resampled array
    :param int dim: Axis over which to do the resampling

    - Output parameters:

    :returns: Resampled array
    :rtype: numpy.ndarray

    ``fftresample(f, L)`` returns a Fourier interpolation of the signal **f**
    to length **L**. If the function is applied to a matrix, it will apply
    to each column.

    ``fftresample(f, L, dim)`` does the same along dimension **dim**.

    If the input signal is *not* a periodic signal (or close to), the
    :func:`dctresample` method gives much better results at the endpoints.

    .. seealso:: :func:`dctresample`,
        :func:`~ltfatpy.fourier.middlepad.middlepad`
    """

    f, L, Ls, W, dim, permutedsize, order = assert_sigreshape_pre(f, L, dim)

    # The 'axis=0' and 'dim=0' below have been added to avoid fft and middlepad
    # being smart about choosing the dimension.
    if np.isrealobj(f):
        L2 = int(np.floor(L/2))+1
        # Note: There is a bug in the call of postpad below in ltfat 2.1.0. for
        # Octave. This bug has been reported and confirmed here:
        # http://sourceforge.net/p/ltfat/bugs/117/
        # This Python port corrects the bug by correclty padding with zeros
        # instead of ones in the Octave version.
        h = ifftreal(postpad(fftreal(f, dim=0), L2, dim=0), L, dim=0)/Ls*L
    else:
        h = np.fft.ifft(middlepad(np.fft.fft(f, axis=0), L, dim=0),
                        axis=0)/Ls*L

    h = assert_sigreshape_post(h, dim, permutedsize, order)

    return h



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of Root Mean Square calculation

Ported from ltfat_2.1.0/sigproc/rms.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np
from numpy import linalg as LA

from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post


def rms(f, ac=False, dim=None):
    """RMS value of signal

    - Usage:

        | ``y = rms(f)``

    - Input parameters:

    :param numpy.ndarray f: Input signal
    :param bool ac: ``True`` if calculation should only consider the AC
        component of the signal (i.e. the mean is removed). ``False`` by
        default.
    :param int dim: Dimension along which norm is applied (first non-singleton
        dimension as default)

    - Output parameters:

    :returns: RMS value
    :rtype: float

    ``rms(f)`` computes the RMS (Root Mean Square) value of a finite sampled
    signal sampled at a uniform sampling rate. This is a vector norm
    equal to the :math:`l^2` averaged by the length of the signal.

    If the input is a matrix or ND-array, the RMS is computed along the
    first (non-singleton) dimension, and a vector of values is returned.

    The RMS value of a signal ``f`` of length ``N`` is computed by

    ..                       N
       rms(f) = 1/sqrt(N) ( sum |f(n)|^2 )^(1/2)
                            n=1

    .. math::
        rms(f) = \\frac{1}{\\sqrt N} \\left( \\sum_{n=1}^N |f(n)|^2
        \\right)^{\\frac{1}{2}}

    """
    # It is better to use 'norm' instead of explicitly summing the squares, as
    # norm (hopefully) attempts to avoid numerical overflow.

    (f, L, _unused, W, dim, permutedsize, order) = \
        assert_sigreshape_pre(f, dim=dim)

    permutedshape = (1,) + permutedsize[1:]
    y = np.zeros(permutedshape)
    if W == 1:
        if ac:
            y[0] = LA.norm(f[:, 0] - np.mean(f[:, 0])) / np.sqrt(L)
        else:
            y[0] = LA.norm(f[:, 0]) / np.sqrt(L)
    else:
        if ac:
            for ii in range(W):
                y[0, ii] = LA.norm(f[:, ii] - np.mean(f[:, ii])) / np.sqrt(L)
        else:
            for ii in range(W):
                y[0, ii] = LA.norm(f[:, ii]) / np.sqrt(L)

    y = assert_sigreshape_post(y, dim, permutedshape, order)
    return y



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of idgt calculation

Ported from ltfat_2.1.0/gabor/idgt.m

.. moduleauthor:: Denis Arrivault
"""

import numpy as np
from ltfatpy.gabor.dgtlength import dgtlength
from ltfatpy.gabor.gabwin import gabwin
from ltfatpy.comp.comp_isepdgt import comp_isepdgt
from ltfatpy.tools.postpad import postpad
from ltfatpy.comp.comp_sigreshape_post import comp_sigreshape_post


def idgt(coef, g, a, Ls=None, pt='freqinv'):
    """Inverse discrete Gabor transform

    - Usage:

        | ``(f, g) = idgt(c, g, a)``
        | ``(f, g) = idgt(c, g, a, Ls)``
        | ``(f, g) = idgt(c, g, a, Ls, pt)``

    - Input parameters:

    :param numpy.ndarray c: Array of coefficients
    :param g: Window function
    :param int a: Length of time shift
    :param int Ls: Length of signal
    :param str pt: 'freqinv' or 'timeinv'. Default is 'freqinv'.
    :type g: str, dict or numpy.ndarray

    - Output parameters:

    :return: Signal (dtype = complex128)
    :rtype: numpy.ndarray

    ``idgt(c, g, a)`` computes the Gabor expansion of the input coefficients
    **c** with respect to the window **g** and time shift **a**. The number of
    channels is deduced from the size of the coefficients **c**.

    ``idgt(c, g, a, Ls)`` does as above but cuts or extends **f** to length
    **Ls**.

    ``(f, g)=idgt(...)`` additionally outputs the window used in the
    transform. This is useful if the window was generated from a description
    in a string or cell array.

    For perfect reconstruction, the window used must be a dual window of the
    one used to generate the coefficients.

    The window **g** may be a vector of numerical values, a text string or a
    cell array. See the help of :func:`~ltfatpy.gabor.gabwin` for more details.

    If **g** is a row vector, then the output will also be a row vector. If
    **c** is 3-dimensional, then ``idgt`` will return a matrix consisting of
    one column vector for each of the TF-planes in **c**.

    Assume that ``f=idgt(c, g, a, L)`` for an array **c** of size
    :math:`M \\times N`. Then the following holds for :math:`k=0,\\ldots,L-1`:

    .. math::

        f(l+1) = \\sum_{n=0}^{N-1}\\sum_{m=0}^{M-1}c(m+1,n+1)e^{2\\pi iml/M}
            g(l-an+1)

    - Additional parameters:

        ``idgt`` takes the following keyword at the end of the line of input
        arguments:

        pt='freqinv'
            Compute a DGT using a frequency-invariant phase. This
            is the default convention described above.
        pt='timeinv'
            Compute a DGT using a time-invariant phase. This
            convention is typically used in FIR-filter algorithms.

    - Examples:

        The following example demonstrates the basic principles for getting
        perfect reconstruction (short version)::

            >>> from ltfatpy import greasy
            >>> from ltfatpy import dgt
            >>> f = greasy()[0]   # Input test signal
            >>> a = 32  # time shift
            >>> M = 64  # frequency shift
            >>> gs = {'name': 'blackman', 'M': 128}  # synthesis window
            >>> # analysis window
            >>> ga = {'name' : ('dual', gs['name']), 'M' : 128}
            >>> (c, Ls) = dgt(f, ga, a, M)[0:2]  # analysis
            >>> # ... do interesting stuff to c at this point ...
            >>> r = idgt(c, gs, a, Ls)[0]  # synthesis
            >>> np.linalg.norm(f-r) < 1e-10 # test
            True

    .. seealso:: :func:`~ltfatpy.gabor.dgt.dgt`,
        :func:`~ltfatpy.gabor.gabwin.gabwin`, :func:`dwilt`,
        :func:`~ltfatpy.gabor.gabtight.gabtight`
  """
    if (not isinstance(g, np.ndarray) and not isinstance(g, str) and
       not isinstance(g, dict)):
        raise TypeError('g must be a numpy.array or str or dict.')

    if (isinstance(g, np.ndarray) and g.size < 2):
        raise ValueError('g must be a vector (you probably forgot to supply' +
                         ' the window function as input parameter.)')

    # Define initial value for flags and key/value pairs.
    if coef.ndim < 2:
        raise ValueError('coef must have at least 2 dimensions')
    M = coef.shape[0]
    N = coef.shape[1]
    if coef.ndim > 2:
        W = coef.shape[2]
    else:
        W = 1

    if not isinstance(a, int):
        raise TypeError('a must be an integer')

    L = N * a
    Ltest = dgtlength(L, a, M)
    if Ltest != L:
        ValueError('Incorrect size of coefficient array or "a" parameter. ' +
                   ' See the help of DGTLENGTH for the requirements.')
    # verify pt
    if pt == 'timeinv':
        pt = 1
    elif pt == 'freqinv':
        pt = 0
    else:
        mes = "pt (" + str(pt) + ") argument should be 'timeinv' or 'freqinv'."
        raise ValueError(mes)

    # Determine the window
    gnum = gabwin(g, a, M, L)[0]
    f = comp_isepdgt(coef, gnum, a, pt)

    # Cut or extend f to the correct length, if desired.
    if Ls is not None:
        f = postpad(f, Ls)
    else:
        Ls = L

    f = comp_sigreshape_post(f, Ls, 0, (0, W))
    return (f, gnum)

if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of greasy signal loading

Ported from ltfat_2.1.0/signals/greasy.m

.. moduleauthor:: Denis Arrivault,
                  Florent Jaillet
"""

from __future__ import print_function, division

from scipy.io.wavfile import read as wavread
import importlib_resources  # not importlib.resources

import numpy as np


def greasy():
    """Load the 'greasy' test signal

    - Usage:

        | ``(s, fs) = greasy()``

    - Output parameters:

    :returns: ``(s, fs)``
    :rtype: tuple

    :var numpy.ndarray s: 'greasy' signal
    :var int fs: sampling frequency in Hz

    ``greasy`` loads the 'greasy' signal. It is a recording of a woman
    pronouncing the word "greasy".

    The signal is 5880 samples long and recorded at 16 kHz with around 11
    bits of effective quantization.

    The signal has been scaled to not produce any clipping when
    played. To get integer values use ``numpy.rint(greasy()[0]*2048.)``.

    The signal was obtained from Wavelab
    (`<http://www-stat.stanford.edu/~wavelab/>`_), it is a part of the first
    sentence of the TIMIT speech corpus "She had your dark suit in greasy
    wash water all year":
    `<http://www.ldc.upenn.edu/Catalog/CatalogEntry.jsp?catalogId=LDC93S1>`_.

    - Examples:

        Plot of 'greasy' in the time-domain:

        >>> import numpy as np
        >>> import matplotlib.pyplot as plt
        >>> from ltfatpy import greasy
        >>> _ = plt.plot(np.arange(5880.)/16000., greasy()[0]);
        >>> _ = plt.xlabel('Time (seconds)')
        >>> _ = plt.ylabel('Amplitude')
        >>> plt.show()

        Plot of 'greasy' in the time-frequency-domain:

        >>> import matplotlib.pyplot as plt
        >>> from ltfatpy import greasy, sgram
        >>> _ = sgram(greasy()[0], 16000., 90.)
        >>> plt.show()

    .. image:: images/greasy_1.png
       :width: 700px
       :alt: time domain image
       :align: center
    .. image:: images/greasy_2.png
       :width: 600px
       :alt: spectrogram image
       :align: center

    - References:
        :cite:`mazh93`
    """

    # f = resource_stream(__name__, "greasy.wav")
    # ref = importlib.resources.files(__name__).joinpath("greasy.wav")
    ref = importlib_resources.files(__package__).joinpath("greasy.wav")
    with ref.open('rb') as f:
       fs, s = wavread(f)
    s = s.astype(np.float64) / 2.**15.
    return (s, fs)

if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the assert_sigreshape_pre function

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import unittest
import random
import numpy as np
from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre as asp
import functools


class TestAssertSigReshapePre(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print("Start TestAssertSigReshapePre")

    # Called after the tests.
    def tearDown(self):
        print("Test done")

    def test_default(self):
        """Basic usual cases"""
        L = random.randint(5, 200)
        f = np.arange(0, L, dtype=np.float64)
        (g, L, Ls, W, dim, _permutedshape, _order) = asp(f)
        mess = "\nL = {0:d}, Ls = {1:d}, W = {2:d}, dim = {3:d}, g.shape = "
        mess += str(g.shape) + ", f.shape = " + str(f.shape)
        mess = mess.format(L, Ls, W, dim)
        np.testing.assert_array_equal(f, g.reshape(L), mess)
        self.assertEqual(L, L, mess)
        self.assertEqual(W, 1, mess)
        self.assertEqual(dim, 0, mess)

        f = f.reshape(L, 1)
        (g, L, Ls, W, dim, _permutedshape, _order) = asp(f)
        mess = "\nL = {0:d}, Ls = {1:d}, W = {2:d}, dim = {3:d}, g.shape = "
        mess += str(g.shape) + ", f.shape = " + str(f.shape)
        mess = mess.format(L, Ls, W, dim)
        np.testing.assert_array_equal(f, g, mess)
        self.assertEqual(L, L, mess)
        self.assertEqual(W, 1, mess)
        self.assertEqual(dim, 0, mess)

    def test_list(self):
        """Basic signal in a list"""
        L = random.randint(5, 200)
        f = [x for x in range(0, L)]
        (g, L, Ls, W, dim, _permutedshape, _order) = asp(f)
        mess = "\nL = {0:d}, Ls = {1:d}, W = {2:d}, dim = {3:d}, g.shape = "
        mess += str(g.shape) + ", len(f) = " + str(len(f))
        mess = mess.format(L, Ls, W, dim)
        np.testing.assert_array_equal(f, g.reshape(L), mess)
        self.assertEqual(L, L, mess)
        self.assertEqual(W, 1, mess)
        self.assertEqual(dim, 0, mess)
        self.assertRaises(TypeError, asp, {"a": 1})
        self.assertRaises(TypeError, asp, f, L, (1, 0))
        self.assertRaises(TypeError, asp, f, L, -4)
        self.assertRaises(TypeError, asp, f, 5.5)

    def test_twodim(self):
        """Many channels signals"""
        shapef = tuple([random.randint(5, 20) for _ in range(2)])
        L = functools.reduce(lambda x, y: x * y, shapef)
        f = np.arange(0, L, dtype=np.complex128)
        f = f.reshape(shapef)
        (g, L, Ls, W, dim, _permutedshape, _order) = asp(f)
        mess = "\nL = {0:d}, Ls = {1:d}, W = {2:d}, dim = {3:d}, g.shape = "
        mess += str(g.shape) + ", f.shape = " + str(f.shape)
        mess = mess.format(L, Ls, W, dim)
        np.testing.assert_array_equal(f, g, mess)
        self.assertEqual(L, L, mess)
        self.assertEqual(W, shapef[1], mess)
        self.assertEqual(dim, 0, mess)
        L = random.randint(5, 200)
        f = np.arange(0, L, dtype=np.float64)
        fcol = f.reshape((1, L))
        (g, L, Ls, W, dim, _permutedshape, _order) = asp(fcol)
        mess = "\nL = {0:d}, Ls = {1:d}, W = {2:d}, dim = {3:d}, g.shape = "
        mess += str(g.shape) + ", f.shape = " + str(fcol.shape)
        mess = mess.format(L, Ls, W, dim)
        np.testing.assert_array_equal(f, g.squeeze(), mess)
        self.assertEqual(L, L, mess)
        self.assertEqual(W, 1, mess)
        self.assertEqual(dim, 1, mess)


if __name__ == "__main__":
    suite = unittest.TestLoader().loadTestsFromTestCase(TestAssertSigReshapePre)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of Sampled, periodized hyperbolic secant calculation

Ported from ltfat_2.1.0/fourier/psech.m

.. moduleauthor:: Denis Arrivault
"""

import numpy as np


def psech(L, tfr=None, s=None, **kwargs):
    """Sampled, periodized hyperbolic secant

    - Usage:

        | ``(g, tfr) = psech(L)``
        | ``(g, tfr) = psech(L, tfr)``
        | ``(g, tfr) = psech(L, s=...)``

    - Input parameters:

    :param int L: length of vector.
    :param float tfr: ratio between time and frequency support.
    :param int s: number of samples (equivalent to :math:`tfr=s^2/L`)

    - Output parameters:

    :returns: ``(g, tfr)``
    :rtype: tuple
    :var numpy.ndarray g: periodized hyperbolic cosine
    :var float tfr: calculated ratio between time and frequency support

    ``psech(L,tfr)`` computes samples of a periodized hyperbolic secant.
    The function returns a regular sampling of the periodization
    of the function :math:`sech(\\pi\\cdot x)`

    The returned function has norm equal to 1.

    The parameter **tfr** determines the ratio between the effective support
    of **g** and the effective support of the DFT of **g**. If **tfr** > 1
    then **g** has a wider support than the DFT of **g**.

    ``psech(L)`` does the same setting than **tfr** = 1.

    ``psech(L,s)`` returns a hyperbolic secant with an effective support of
    **s** samples. This means that approx. 96% of the energy or 74% or the
    area under the graph is contained within **s** samples. This is
    equivalent to ``psech(L,s^2/L)``.

    ``(g,tfr) = psech( ... )`` returns the time-to-frequency support ratio.
    This is useful if you did not specify it (i.e. used the **s** input
    format).

    The function is whole-point even.  This implies that
    ``fft(psech(L,tfr))`` is real for any **L** and **tfr**.

    If this function is used to generate a window for a Gabor frame, then
    the window giving the smallest frame bound ratio is generated by
    ``psech(L,a*M/L)``.

    - Examples:

        This example creates a ``psech`` function, and demonstrates that it
        is its own Discrete Fourier Transform:

        >>> import numpy as np
        >>> import numpy.linalg as nla
        >>> g = psech(128)[0] # DFT invariance: Should be close to zero.
        >>> diff = nla.norm(g-np.fft.fft(g)/np.sqrt(128))
        >>> np.abs(diff) < 10e-10
        True

    .. seealso:: :func:`~ltfatpy.fourier.pgauss.pgauss`, :func:`pbspline`,
        :func:`pherm`

    - References:
        :cite:`jast02-1`
    """
    if not isinstance(L, int):
        raise TypeError('L must be an integer')

    if s is not None:
        if not isinstance(s, int):
            raise TypeError('s must be an integer')
        tfr = float(s**2 / L)
    elif tfr is None:
        tfr = 1

    safe = 12
    g = np.zeros(L)
    sqrtl = np.sqrt(L)
    w = tfr

    # Outside the interval [-safe,safe] then sech(pi*x) is numerically zero.
    nk = np.ceil(safe / np.sqrt(L / np.sqrt(w)))

    lr = np.arange(L)
    for k in np.arange(-nk, nk+1):
        g = g + (1 / np.cosh(np.pi * (lr / sqrtl - k * sqrtl) / np.sqrt(w)))

    # Normalize it.
    g = g * np.sqrt(np.pi / (2 * np.sqrt(L*w)))
    return(g, tfr)

if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of fir windows calculation

Ported from ltfat_2.1.0/sigproc/firwin.m

.. moduleauthor:: Denis Arrivault
"""

import numpy as np
from numpy.lib.scimath import sqrt as npsqrt

from ltfatpy.sigproc.normalize import normalize


def firwin(name, M=0, x=None, **kwargs):
    """ Returns a FIR window of length *M* of type *name*.

    - Usage:

        | ``(g, info) = firwin(name, M)``
        | ``(g, info) = firwin(name, M,...)``
        | ``(g, info) = firwin(name, x)``

    - Input parameters:

    :param str name: is the window.
    :param int M: is the length of the window
    :param numpy.ndarray x: is a points vector (default = None)

    - Output parameters:

    :returns: ``(g, info)``
    :rtype: tuple

    :var numpy.ndarray g: window values array
    :var dict info: the information dictionary

    All windows are symmetric and generate zero delay and zero phase
    filters. They can be used for the Wilson and WMDCT transform, except
    when noted otherwise.

    ``firwin(name, x=numpy.array(...))`` where **x** is a vector will sample
    the window definition as the specified points. The normal sampling
    interval for the windows is :math:`-.5< x <.5`.

    In the following PSL means "Peak Sidelobe level", and the main lobe
    width is measured in normalized frequencies.

    If a window **g** forms a "partition of unity" (PU) it means specifically
    that

    .. math::

        g + \\mbox{fftshift}(g) = \\mathbf{1}_L

    A PU can only be formed if the window length is even, but some windows
    may work for odd lengths anyway.

    If a window is the square root of a window that forms a PU, the window
    will generate a tight Gabor frame / orthonormal Wilson/WMDCT basis if
    the number of channels is less than **M**.

    - The windows available are:

        ===========  =========================================================
        'hann'       von Hann window. Forms a PU. The Hann window has a
                     mainlobe with of 8/M, a PSL of -31.5 dB and decay rate
                     of 18 dB/Octave.

        'sine'       Sine window. This is the square root of the Hanning
                     window. The sine window has a mainlobe width of 8/M,
                     a  PSL of -22.3 dB and decay rate of 12 dB/Octave.

                        - Aliases: `'cosine'`, `'sqrthann'`

        'rect'       (Almost) rectangular window. The rectangular window has a
                     mainlobe width of 4/M, a  PSL of -13.3 dB and decay
                     rate of 6 dB/Octave. Forms a PU if the order is odd.

                        - Alias: `'square'`

        'tria'       (Almost) triangular window. Forms a PU.

                        - Alias: `'bartlett'`

        'sqrttria'   Square root of the triangular window.

        'hamming'    Hamming window. Forms a PU that sums to 1.08 instead
                     of 1.0 as usual. The Hamming window has a
                     mainlobe width of 8/M, a  PSL of -42.7 dB and decay
                     rate of 6 dB/Octave. This window should not be used for
                     a Wilson basis, as a reconstruction window cannot be
                     found by `wildual`.

        'blackman'   Blackman window. The Blackman window has a
                     mainlobe width of 12/M, a PSL of -58.1 dB and decay
                     rate of 18 dB/Octave.

        'blackman2'  Alternate Blackman window. This window has a
                     mainlobe width of 12/M, a PSL of -68.24 dB and decay
                     rate of 6 dB/Octave.

        'itersine'   Iterated sine window. Generates an orthonormal
                     Wilson/WMDCT basis. This window is described in
                     Wesfreid and Wickerhauser (1993) and is used in the
                     ogg sound codec.

                        - Alias: `'ogg'`

        'nuttall'    Nuttall window. The Nuttall window has a
                     mainlobe width of 16/M, a PSL of -93.32 dB and decay
                     rate of 18 dB/Octave.
        'nuttall10'  2-term Nuttall window with 1 continuous derivative.

                        - Alias: `'hann'`, `'hanning'`.

        'nuttall01'  2-term Nuttall window with 0 continuous derivatives.
                     This is a slightly improved Hamming window. It has a
                     mainlobe width of 8/M, a  PSL of -43.19 dB and decay
                     rate of 6 dB/Octave.

        'nuttall20'  3-term Nuttall window with 3 continuous derivatives.
                     The window has a mainlobe width of 12/M, a PSL of
                     -46.74 dB and decay rate of 30 dB/Octave.

        'nuttall11'  3-term Nuttall window with 1 continuous derivative.
                     The window has a mainlobe width of 12/M, a PSL of
                     -64.19 dB and decay rate of 18 dB/Octave.
        'nuttall02'  3-term Nuttall window with 0 continuous derivatives.
                     The window has a mainlobe width of 12/M, a PSL of
                     -71.48 dB and decay rate of 6 dB/Octave.

        'nuttall30'  4-term Nuttall window with 5 continuous derivatives.
                     The window has a mainlobe width of 16/M, a PSL of
                     -60.95 dB and decay rate of 42 dB/Octave.

        'nuttall21'  4-term Nuttall window with 3 continuous derivatives.
                     The window has a mainlobe width of 16/M, a PSL of
                     -82.60 dB and decay rate of 30 dB/Octave.

        'nuttall12'  4-term Nuttall window with 1 continuous derivatives.

                        - Alias: `'nuttall'`.

        'nuttall03'  4-term Nuttall window with 0 continuous derivatives.
                     The window has a mainlobe width of 16/M, a PSL of
                     -98.17 dB and decay rate of 6 dB/Octave.
        ===========  =========================================================

    - Additional keywords arguments:

        ``firwin`` understands the following keyword arguments at the end
        of the list of input:

        **shift** = s
             Shift the window by :math:`s` samples. The value can be a
             fractional number.
        **centering** = 'wp' or 'hp'
             Point even output type : whole or half point even.
             Whole point even is the default. It corresponds to a shift
             of :math:`s=0`.
             Half point even is the convention of most Matlab filter
             routines. It corresponds to a shift of :math:`s=-.5`
        **taper** = t
             Extend the window by a flat section in the middle. The
             argument t is the ratio of the rising and falling
             parts as compared to the total length of the
             window. The default value of 1 means no
             tapering. Accepted values lie in the range from 0 to 1.

    Additionally, ``firwin`` accepts flags to normalize the output.
    Please see the help of :py:meth:`~ltfatpy.sigproc.normalize`.
    Default is to use no normalization.
    For filtering in the time-domain, a normalization of `'1'` or `'area'`
    is preferable.

    .. seealso:: :func:`~ltfatpy.fourier.pgauss.pgauss`, :func:`pbspline`,
        :func:`firkaiser`, :func:`~ltfatpy.sigproc.normalize.normalize`

    - References:
        :cite:`opsc89,harris1978,nuttall1981,wesfreid1993`
    """
    info = {}
    g = None
    if not isinstance(name, str):
        raise TypeError("First argument must be a string containing the name" +
                        " of a window")
    if isinstance(M, float):
        M = int(M)
    if not isinstance(M, int):
        raise TypeError("Second argument must be an integer containing the" +
                        " length of the window")
    # Always set to this
    info['isfir'] = True
    # Default values, may be overwritten later in the code
    info['ispu'] = False
    info['issqpu'] = False
    name = name.lower()

    # Define initial value for flags and key/value pairs.
    shift = 0
    if 'shift' in kwargs:
        shift = kwargs['shift']
    if 'centering' in kwargs:
        if kwargs['centering'] == 'hp':
            shift = .5
    taper = 1
    if 'taper' in kwargs:
        if kwargs['taper'] < 1 and kwargs['taper'] >= 0:
            taper = kwargs['taper']

    if M == 0 and x is None:
        return (g, info)

    Xdefined = True

    if x is None:
        # Deal with tapering
        Xdefined = False
        if taper == 0:
            # Window is only tapering, do this and bail out, because subsequent
            # code may fail.
            return (np.ones(M), info)
        # Modify M to fit with tapering
        Morig = M
        M = int(np.round(M * taper))
        Mtaper = Morig - M

        p1 = int(np.round(Mtaper / 2))
        p2 = Mtaper - p1

        # Switch centering if necessary
        if p1 != p2:
            if shift == 0:
                shift = .5
            elif shift == .5:
                shift = 1

        # This is the normally used sampling vector.
        if (M % 2) == 0:  # For even M the sampling interval is [-.5,.5-1/M]
            # Matlab : [0:1/M:.5-1/M,-.5:1/M:-1/M]'
            x = np.r_[0:.5:1/M, -.5:0:1/M]
        else:  # For odd M the sampling interval is [-.5+1/(2M),.5-1/(2M)]
            # Matlab : x = [0:1/M:.5-.5/M,-.5+.5/M:1/M:-1/M]'
            x = np.r_[0:.5:1/M, -.5+.5/M:-.5/M:1/M]

        x = x + shift / M
    else:
        if M != 0 and M != len(x):
            raise ValueError("M should be equel to len(x).")
        M = len(x)

    do_sqrt = False

    if name in {'hanning', 'hann', 'nuttall10'}:
        g = (.5 + .5 * np.cos(2 * np.pi * x))
        info['ispu'] = True
    elif name in {'sine', 'cosine', 'sqrthann'}:
        g = firwin('hanning', M, **kwargs)[0]
        info['issqpu'] = True
        do_sqrt = True
    elif name == 'hamming':
        g = 0.54 + 0.46 * np.cos(2 * np.pi * x)
        # This is the definition taken from the Harris paper
        # elif name is 'hammingacc'
        # g = 25/46 + 21/46 * np.cos(2 * np.pi *x)
    elif name == 'nuttall01':
        g = 0.53836 + 0.46164 * np.cos(2 * np.pi * x)
    elif name in {'square', 'rect'}:
        g = np.asarray(np.abs(x) < .5, dtype='f8')
    elif name in {'tria', 'triangular', 'bartlett'}:
        g = 1.0 - 2.0 * np.abs(x)
        info['ispu'] = True
    elif name == 'sqrttria':
        arg_centering = {}
#         if 'shift' in kwargs:
#             arg_centering['shift'] = kwargs['shift']
        if 'centering' in kwargs:
            arg_centering['centering'] = kwargs['centering']
        g = firwin('tria', M, **arg_centering)[0]
        info['issqpu'] = True
        do_sqrt = True
    # Rounded version of blackman2
    elif name == 'blackman':
        g = 0.42 + 0.5 * np.cos(2 * np.pi * x) + 0.08 * np.cos(4 * np.pi * x)
    elif name == 'blackman2':
        g = 7938/18608 + 9240/18608 * np.cos(2 * np.pi * x) + 1430/18608 * \
            np.cos(4 * np.pi * x)
    elif name in {'nuttall', 'nuttall12'}:
        g = 0.355768 + 0.487396 * np.cos(2 * np.pi * x) + 0.144232 * \
            np.cos(4 * np.pi * x) + 0.012604 * np.cos(6 * np.pi * x)
    elif name in {'itersine', 'ogg'}:
        g = np.sin(np.pi / 2 * np.cos(np.pi * x)**2)
        info['issqpu'] = True
    elif name == 'nuttall20':
        g = 3/8 + 4/8 * np.cos(2 * np.pi * x) + 1/8 * np.cos(4 * np.pi * x)
    elif name == 'nuttall11':
        g = 0.40897 + 0.5 * np.cos(2 * np.pi * x) + 0.09103 * \
            np.cos(4 * np.pi * x)
    elif name == 'nuttall02':
        g = 0.4243801 + 0.4973406 * np.cos(2 * np.pi * x) + 0.0782793 * \
            np.cos(4 * np.pi * x)
    elif name == 'nuttall30':
        g = 10/32 + 15/32 * np.cos(2 * np.pi * x) + 6/32 * \
            np.cos(4 * np.pi * x) + 1/32 * np.cos(6 * np.pi * x)
    elif name == 'nuttall21':
        g = 0.338946 + 0.481973 * np.cos(2 * np.pi * x) + 0.161054 * \
            np.cos(4 * np.pi * x) + 0.018027 * np.cos(6 * np.pi * x)
    elif name == 'nuttall03':
        g = 0.3635819 + 0.4891775 * np.cos(2 * np.pi * x) + 0.1365995 * \
            np.cos(4 * np.pi * x) + 0.0106411 * np.cos(6 * np.pi * x)
    else:
        raise ValueError('Unknown window: ' + name + '.')

    # Force the window to 0 outside (-.5,.5)
    g = g * np.array(np.abs(x) < .5, dtype=int)

    if not Xdefined and taper < 1:
        # Perform the actual tapering.
        g = np.hstack((np.ones(p1), g, np.ones(p2)))
    # Do sqrt if needed.
    if do_sqrt:
        g = npsqrt(g)

    if 'norm' in kwargs:
        g = normalize(g, norm=kwargs['norm'])[0]
#     else:
#         g = normalize(g, norm = 'null')[0]

    return (g, info)

if __name__ == '__main__':  # pragma: no cover
    (g, info) = firwin(name='sine', M=18, centering='wp')
    print(g)
    print(info)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""This module contains sampled, periodized Gaussian window function

Ported from ltfat_2.1.0/fourier/pgauss.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

from ltfatpy.comp import comp_pgauss as pg
from ltfatpy.sigproc.normalize import normalize


def pgauss(L, tfr=1.0, fs=0.0, width=0.0, bw=0.0, c_f=0.0, centering='wp',
           delay=0.0, norm='2', **kwargs):
    """Sampled, periodized Gaussian

    - Usage:
        | ``(g, tfr) =  pgauss(L)``
        | ``(g, tfr) =  pgauss(L, tfr, fs, width, bw, c_f, centering, delay,
            norm)``

    - Input parameters:

    :param int L: Length of the output vector.
    :param float tfr: determines the ratio between the effective support
        of **g** and the effective support of the DFT of **g**. If
        :math:`tfr>1` then **g** has a wider support than the DFT of **g**.
        Default is 1.
    :param float fs: Use a sampling rate of **fs** Hz as unit for specifying
        the width, bandwidth, centre frequency and delay of the Gaussian.
        Default is :math:`fs=0` which indicates to measure everything in
        samples.
    :param float width: Set the width of the Gaussian such that it has an
        effective support of **width** samples. This means that
        approx. 96% of the energy or 79% of the area under the graph is
        contained within **width** samples. This corresponds to a -6 db
        cutoff. This is equivalent to calling ``pgauss(L,tfr = s^2/L)``.
        Default is zero, unset.
    :param float bw: As for the **width** argument, but specifies the width
        in the frequency domain. The bandwidth is measured in normalized
        frequencies, unless the **fs** value is given. Default is zero, unset.
    :param float c_f: Set the centre frequency of the Gaussian to **cf**.
        Default is zero.
    :param str centering: "wp" means output is whole point even. This is the
        default. Setting it to **hp** means output is half point even which
        is the case for the most Matlab filters routines.
    :param float delay: Delay the output by **delay**. Default is zero.
    :param str norm: normalization to apply (default is L2)


    - Normalization types :

                 +-------+---------+
                 | Name  | Norms   |
                 +=======+=========+
                 | '1'   | L1-norm |
                 +-------+---------+
                 | '2'   | L2-norm |
                 +-------+---------+

    - Output parameters:

    :returns: ``(g, tfr)``
    :rtype: tuple
    :var numpy array g: window array of length **L**
    :var int tfr: ratio between the effective support of **g** and the
        effective support of the DFT of **g**.

    ``pgauss(L,tfr)`` samples of a periodized Gaussian. The function returns
    a numpy array **g** containing a regular sampling of the periodization of
    the function :math:`\\exp(-\\pi*(x^2/tfr))`.

    The :math:`l^2` norm of the returned Gaussian is equal to 1.

    The function is whole-point even. This implies that
    ``fft(pgauss(L, tfr))`` is real for any **L** and **tfr**. The DFT of
    **g** is equal to ``pgauss(L, 1/tfr)``.

    If this function is used to generate a window for a Gabor frame, then
    the window giving the smallest frame bound ratio is generated by
    ``pgauss(L, a*M/L)``.

    - Examples:

    >>> import numpy as np
    >>> import matplotlib.pyplot as plt
    >>> from ltfatpy import sgram
    >>> # This example creates a Gaussian function, and demonstrates that it is
    >>> # its own Discrete Fourier Transform:
    >>> g = pgauss(128)[0]
    >>> # Test of DFT invariance: Should be close to zero.
    >>> np.linalg.norm(g - np.fft.fft(g)/np.sqrt(128)) <= 1e-10
    True
    >>> # The next plot shows the Gaussian in the time domain:
    >>> _ = plt.plot(np.fft.fftshift(g))
    >>> plt.show()
    >>> # The next plot shows the Gaussian in the time-frequency plane:
    >>> _ = sgram(g, nf=True, tc=True, normalization='lin')
    >>> plt.show()

    .. image:: images/pgauss_1.png
       :width: 700px
       :alt: Gaussian in the time domain
       :align: center
    .. image:: images/pgauss_2.png
       :width: 700px
       :alt: Gaussian in the time-frequency plane
       :align: center

    .. note:: **g** is real if **c_f** = 0 and complex if not.

    .. seealso:: :func:`~ltfatpy.gabor.dgtlength.dgtlength`,
        :func:`~ltfatpy.fourier.psech.psech`,
        :func:`~ltfatpy.sigproc.firwin.firwin`, :func:`pbspline`,
        :func:`~ltfatpy.sigproc.normalize.normalize`
    """
    cent = 0
    if centering == 'hp':
        cent = 0.5
    elif centering != 'wp':
        raise ValueError("centering should be 'wp' or 'hp'")
    if (type(L) is not int or L <= 0):
        raise TypeError("L should be non null integer")
    if(type(tfr) is int):
        tfr = float(tfr)
    if (type(tfr) is not float):
        raise TypeError("tfr should be float")

    if fs == 0:
        if width != 0:
            tfr = width**2 / L
        if bw != 0:
            tfr = L / (bw * L/2)**2
    else:
        if width != 0:
            tfr = (width * fs)**2 / L
        if bw != 0:
            tfr = L / (bw / fs * L)**2
        delay = delay*fs
        c_f = c_f/fs*L

    g = pg.comp_pgauss(L, tfr, cent-delay, c_f)
    g = normalize(g, norm)[0]
    return(g, tfr)

if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the postpad function

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import unittest
import numpy as np

from ltfatpy.tools.postpad import postpad


class TestPostpad(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print('\nStart TestPostpad')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_working(self):
        """Check that postpad works as expected
        """
        inputs = {}
        N = 5
        inputs['x'] = np.arange(N, dtype=float)

        # testing padding
        inputs['L'] = 10
        y = postpad(**inputs)
        msg = ('Wrong shape in postpad output in the padding case using '
               'inputs ' + str(inputs))
        self.assertEqual(y.shape, (inputs['L'],), msg)
        for val_x, val_y in zip(inputs['x'], y[:N]):
            msg = ('Wrong output value in postpad, y should be equal to x in '
                   'the non-padded part. Inputs are ' + str(inputs))
            self.assertEqual(val_x, val_y, msg)
        for val in y[N:]:
            msg = ('Wrong output value in postpad, y should be equal to 0. in '
                   'the padded part. Inputs are ' + str(inputs))
            self.assertEqual(val, 0., msg)

        # testing truncation
        inputs['L'] = 3
        y = postpad(**inputs)
        msg = ('Wrong shape in postpad output in the truncation case using '
               'inputs ' + str(inputs))
        self.assertEqual(y.shape, (inputs['L'],), msg)
        for val_x, val_y in zip(inputs['x'][:inputs['L']], y):
            msg = ('Wrong output value in postpad, y should be equal to x in '
                   'the non-truncated part. Inputs are ' + str(inputs))
            self.assertEqual(val_x, val_y, msg)

    def test_default_val(self):
        """Check that expected default values are used
        """
        # check that default value for C is 0.
        inputs = {}
        N = 3
        inputs['x'] = np.ones(N)
        inputs['L'] = N+1
        y = postpad(**inputs)
        msg = 'Wrong default value for C using inputs ' + str(inputs)
        self.assertEqual(y[-1], 0., )

        # check that default dim is the first non-singleton dimension
        N = 3
        inputs['L'] = N+1
        x_shape = (1, 1, N, 5)
        inputs['x'] = np.ones(x_shape)
        y = postpad(**inputs)
        y_exp_shape = (1, 1, inputs['L'], 5)
        msg = 'Wrong default value for dim using inputs ' + str(inputs)
        self.assertEqual(y.shape, y_exp_shape, msg)

    def test_param_C(self):
        """Check that the value of C is taken into account
        """
        inputs = {}
        N = 3
        inputs['C'] = 2.
        inputs['x'] = np.ones(N)
        inputs['L'] = N+1
        y = postpad(**inputs)
        msg = 'Postpad is misusing C with inputs ' + str(inputs)
        self.assertEqual(y[-1], inputs['C'], msg)

    def test_param_dim(self):
        """Check that dim is taken into account
        """
        inputs = {}
        N = 3
        M = 4
        inputs['L'] = 5
        inputs['x'] = np.ones((N, M))
        inputs['dim'] = 0
        y = postpad(**inputs)
        msg = 'postpad is misusing dim with inputs ' + str(inputs)
        self.assertEqual(y.shape, (inputs['L'], M), msg)
        inputs['dim'] = 1
        y = postpad(**inputs)
        msg = 'postpad is misusing dim with inputs ' + str(inputs)
        self.assertEqual(y.shape, (N, inputs['L']), msg)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestPostpad)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module of phase plot

Ported from ltfat_2.1.0/gabor/phaseplot.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.gabor.plotdgt import plotdgt
from ltfatpy.gabor.plotdgtreal import plotdgtreal
from ltfatpy.fourier.fftresample import fftresample
from ltfatpy.gabor.dgt import dgt
from ltfatpy.gabor.dgtreal import dgtreal


def phaseplot(f, fs=None, tfr=1., wlen=None, nf=None, thr=None, fmax=None,
              phase='timeinv', norm='2', **kwargs):
    """Phase plot

    - Input parameters:

    :param numpy.ndarray f: Analyzed signal
    :param float fs: Sampling rate in Hz of the analyzed signal
    :param float tfr: Set the ratio of frequency resolution to time resolution.
        A value ``tfr = 1.0`` is the default. Setting ``tfr > 1.0`` will give
        better frequency resolution at the expense of a worse time resolution.
        A value of ``0.0 < tfr < 1.0`` will do the opposite.
    :param int wlen: Window length. Specifies the length of the window
        measured in samples. See help of :func:`~ltfatpy.fourier.pgauss` on the
        exact details of the window length (parameter width).
    :param bool nf: If ``True``, display negative frequencies, with the
        zero-frequency centered in the middle. For real signals, this will just
        mirror the upper half plane. This is standard for complex signals.
    :param float thr: Keep the coefficients with a magnitude larger than
        **thr** times the largest magnitude. Set the phase of the rest of the
        coefficients to zero. This is useful, because for small amplitude the
        phase values can be meaningless.
    :param float fmax: Display **fmax** as the highest frequency.
    :param str phase: Phase convention for the dgt computation. Use
        phase='timeinv' to display the phase as computed by a time-invariant
        dgt (this is the default) or phase='freqinv' to display the phase as
        computed by a frequency-invariant dgt.
    :param string norm: Window normalization. See parameter **norm** in the
        help of :func:`~ltfatpy.fourier.pgauss`.
    :param `**kwargs`: ``phaseplot`` supports all the optional parameters of
        :func:`~ltfatpy.gabor.tfplot.tfplot`. Please see the help of
        :func:`~ltfatpy.gabor.tfplot.tfplot` for an exhaustive list.

    - Output parameters:

    :returns: The phase data used in the plotting
    :rtype: numpy.ndarray

    ``phaseplot(f)`` plots the phase of **f** using a dgt.

    ``phaseplot`` should only be used for short signals (shorter than the
    resolution of the screen), as there will otherwise be some visual
    aliasing, such that very fast changing areas will look very smooth.
    ``phaseplot`` always calculates the phase of the full time/frequency plane
    (as opposed to :func:`~ltfatpy.gabor.sgram.sgram`), and you therefore risk
    running out of memory for long signals.

    For the best result when using phaseplot, use a circulant color
    map, for instance hsv.

    - Examples:

        The following code shows the phaseplot of a periodic, hyperbolic
        secant visualized using the hsv colormap:

        >>> from matplotlib.pyplot import hsv, show
        >>> from ltfatpy import phaseplot, psech
        >>> hsv()
        >>> _ = phaseplot(psech(200)[0], tc=True, nf=True)
        >>> show()

        The following phaseplot shows the phase of white, Gaussian noise:

        >>> from numpy.random import randn
        >>> from matplotlib.pyplot import hsv, show
        >>> from ltfatpy import phaseplot
        >>> hsv()
        >>> _ = phaseplot(randn(200))
        >>> show()

    .. image:: images/phaseplot_1.png
       :width: 600px
       :alt: phase image of a periodic, hyperbolic secant
       :align: center

    .. image:: images/phaseplot_2.png
       :width: 600px
       :alt: phase image of white, Gaussian noise
       :align: center

    .. seealso:: :func:`~ltfatpy.gabor.phaselock.phaselock`

    - References:
        :cite:`Carmona98practical`
    """

    if not isinstance(f, np.ndarray):
        raise TypeError('f must be a 1D numpy.ndarray')

    if f.ndim > 1:
        raise ValueError('Input must be a vector.')

    # Override the setting from tfplot, because phaseplot only uses the 'lin'
    # plotting.
    kwargs['normalization'] = 'lin'

    if nf is None:
        if np.isrealobj(f):
            nf = False
        else:
            nf = True

    # Downsample
    if fmax:
        if fs:
            resamp = fmax*2./fs
        else:
            resamp = fmax*2./f.shape[0]

        f = fftresample(f, int(round(f.shape[0] * resamp)))
        fs = 2. * fmax

    # Always do the full STFT
    L = f.shape[0]
    a = 1
    M = L

    # Set an explicit window length, if this was specified.
    if wlen:
        tfr = wlen**2 / L

    g = {'name': 'gauss', 'tfr': tfr, 'norm': norm}

    if nf:
        coef = dgt(f, g, a, M, pt=phase)[0]
    else:
        coef = dgtreal(f, g, a, M, pt=phase)[0]

    if thr:
        # keep only the largest coefficients.
        maxc = np.max(np.abs(coef))
        mask = np.abs(coef) < maxc*thr
        coef[mask] = 0.

    coef = np.angle(coef)

    if nf:
        plotdgt(coef, a, fs=fs, **kwargs)
    else:
        plotdgtreal(coef, a, M, fs=fs, **kwargs)

    return coef


if __name__ == '__main__':  # pragma: no cover
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of signal normalization

Ported from ltfat_2.1.0/sigproc/normalize.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np
from numpy import linalg as LA

from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post
from ltfatpy.sigproc.rms import rms
from ltfatpy.gabor.s0norm import s0norm


def normalize(f, norm='2', dim=None):
    """Normalize input signal by specified norm

    - Usage:

        | ``(f, fnorm) = normalize(f)``
        | ``(f, fnorm) = normalize(f, 'area')``
        | ``(f, fnorm) = normalize(f, dim=2)``
        | ...

    :param numpy.ndarray f: Input signal
    :param str norm: Name of the norm to apply
    :param int dim: Dimension along which norm is applied (first non-singleton
                    dimension as default)

    - Output parameters:

    :return: ``(f, fnorm)``
    :rtype: tuple
    :var numpy.ndarray f: normalized signal
    :var numpy.ndarray fnorm: norm of the signal

    ``normalize(f,...)`` will normalize the signal **f** by the specified norm.

    The norm is specified as a string and may be one of:
        ============ ==========================================================
        ``'1'``      Normalize the :math:`l^1` norm to be *1*.
        ``'area'``   Normalize the area of the signal to be *1*. This is
                     exactly the same as ``'1'``.
        ``'2'``      Normalize the :math:`l^2` norm to be *1*. This is the
                     default
        ``'energy'`` Normalize the energy of the signal to be *1*. This is
                     exactly the same as ``'2'``.
        ``'inf'``    Normalize the :math:`l^{\\inf}` norm to be *1*.
        ``'peak'``   Normalize the peak value of the signal to be *1*.
                     This is exactly the same as ``'inf'``.
        ``'rms'``    Normalize the Root Mean Square (RMS) norm of the signal to
                     be *1*.
        ``'s0'``     Normalize the S0-norm to be *1*.
        ``'wav'``    Normalize to the :math:`l^{inf}` norm to be *0.99* to
                     avoid possible clipping introduced by the quantization
                     procedure when saving as a wav file. This only works with
                     floating point data types.
        ``'null'``   Do NOT normalize, output is identical to input.
        ============ ==========================================================

    .. seealso::
        :func:`~ltfatpy.sigproc.rms.rms`,
        :func:`~ltfatpy.gabor.s0norm.s0norm`
    """
    if not isinstance(norm, str):
        raise TypeError('norm should be string.')
    norm = norm.lower()
    f = f.copy()

    (f, _unused, _unused, W, dim, permutedshape, order) = \
        assert_sigreshape_pre(f, dim=dim)

    if np.issubdtype(f.dtype, np.integer) and norm == 'wav':
        raise TypeError('Integer data types are unsupported for wav norm.')

    fnorm = np.zeros((W, ))
    for ii in range(W):
        if norm == '1' or norm == 'area':
            fnorm[ii] = LA.norm(f[:, ii], 1)
            f[:, ii] = f[:, ii] / fnorm[ii]
        elif norm == '2' or norm == 'energy':
            fnorm[ii] = LA.norm(f[:, ii], 2)
            f[:, ii] = f[:, ii] / fnorm[ii]
        elif norm == 'inf' or norm == 'peak':
            fnorm[ii] = LA.norm(f[:, ii], np.inf)
            f[:, ii] = f[:, ii] / fnorm[ii]
        elif norm == 'rms':
            fnorm[ii] = rms(f[:, ii]).item()
            f[:, ii] = f[:, ii] / fnorm[ii]
        elif norm == 's0':
            fnorm[ii] = s0norm(f[:, ii]).item()
            f[:, ii] = f[:, ii] / fnorm[ii]
        elif norm == 'wav':
            if np.issubdtype(f.dtype, np.floating):
                fnorm[ii] = LA.norm(f[:, ii], np.inf)
                f[:, ii] = 0.99 * f[:, ii] / fnorm[ii]
            else:
                raise TypeError("TO DO: Normalizing integer data types not"
                                "supported yet.")

    f = assert_sigreshape_post(f, dim, permutedshape, order)
    return (f, fnorm)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Test of the fftresample function

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import unittest
import numpy as np
from numpy.testing import assert_allclose

from ltfatpy.fourier.fftresample import fftresample
from ltfatpy.tests.datasets.read_ref_mat import read_ref_mat
from ltfatpy.tests.datasets.get_dataset_path import get_dataset_path


class TestFftresample(unittest.TestCase):

    # Called before the tests.
    def setUp(self):
        print('\nStart TestFftresample')

    # Called after the tests.
    def tearDown(self):
        print('Test done')

    def test_shape(self):
        """Check that the output has the expected shape
        """
        shapes = ((4,), (4, 3), (4, 3, 2))
        L_val = (2, 3, 6, 7)
        data_types = ('real', 'complex')
        inputs = {}
        for shape in shapes:
            for data_type in data_types:
                if data_type == 'real':
                    inputs['f'] = np.random.random(shape)
                elif data_type == 'complex':
                    inputs['f'] = (np.random.random(shape) +
                                   1.0j * np.random.random(shape))
                for dim in range(len(shape)):
                    inputs['dim'] = dim
                    for L in L_val:
                        inputs['L'] = L
                        h = fftresample(**inputs)
                        expected_shape = list(shape)
                        expected_shape[dim] = L
                        expected_shape = tuple(expected_shape)
                        msg = ('Wrong shape in the ouput of fftresample with '
                               'inputs ' + str(inputs))
                        self.assertEqual(h.shape, expected_shape, msg)

    def test_same_length(self):
        """Check that the data are unchanged when using the original length
        """
        shape = (16, 17)
        inputs = {}
        data_types = ('real', 'complex')
        for data_type in data_types:
            if data_type == 'real':
                inputs['f'] = np.random.random(shape)
            elif data_type == 'complex':
                inputs['f'] = (np.random.random(shape) +
                               1.0j * np.random.random(shape))
            for dim in range(len(shape)):
                inputs['dim'] = dim
                inputs['L'] = shape[dim]
                h = fftresample(**inputs)
                msg = ('Input and output should be almost equal when '
                       'resampling to the same length in fftresample with '
                       'inputs ' + str(inputs))
                assert_allclose(inputs['f'], h, rtol=1e-12, atol=1e-14,
                                err_msg=msg)

    def test_known(self):
        """Checking fftresample on some known results taken from Octave
        """
        filename = get_dataset_path('fftresample_ref.mat')
        data = read_ref_mat(filename)

        for inputs, outputs in data:
            h = fftresample(**inputs)
            msg = ('Wrong value in output of fftresample with inputs ' +
                   str(inputs))
            assert_allclose(h, outputs[0], rtol=1e-14, err_msg=msg)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestFftresample)
    unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""This module computes the least common multiple

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

from math import copysign
try:
    from math import gcd
except ImportError:
    # fractions.gcd() is deprecated since Python 3.5 and math.gcd() should be
    # used instead, but for backward compatibilty we use fractions.gcd() if
    # math.gcd() is not available
    from fractions import gcd


def lcm(X, Y):
    """Compute the least common multiple

    - Inputs:

    :param int X: The first of the two integers for which to compute the
                  least common multiple
    :param int Y: The second of the two integers for which to compute the
                  least common multiple

    - Outputs:

    :returns: Least common multiple of **X** and **Y**
    :rtype: int

    """

    if (not isinstance(X, int) or not isinstance(Y, int)):
        raise TypeError('X and Y must be int.')

    if Y == 0 or X == 0:
        return 0

    res = X * Y // gcd(X, Y)

    # Note: Some sign modification is needed to match the results in Octave
    # when we have negative values
    res = int(abs(res) * copysign(1, X) * copysign(1, Y))

    return res



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of sampling of continuous Hermite function computation

Ported from ltfat_2.1.0/comp/comp_hermite.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np
import math


def comp_hermite(n, x):
    """ Compute sampling of continuous Hermite function.

    - Usage:

        | ``y = comp_hermite(n, x)``

    `comp_hermite(n, x)` evaluates the n-th Hermite function at the vector
    **x**. The function is normalized to have the :math:`L^2` norm
    on :math:`]-\\infty,\\infty[` equal to one.

    A minimal effort is made to avoid underflow in recursion.
    If used to evaluate the Hermite quadratures, it works for n <= 2400
    """
    rt = 1 / math.sqrt(math.sqrt(math.pi))

    if n == 0:
        y = rt * np.exp(-0.5 * x**2)
    if n == 1:
        y = rt * math.sqrt(2) * x * np.exp(-0.5 * x**2)

    # if n > 2, conducting the recursion.
    if n >= 2:
        ef = np.exp(-0.5 * (x**2) / (n+1))
        tmp1 = rt * ef
        tmp2 = rt * math.sqrt(2) * x * (ef**2)
        for k in range(2, n+1):
            y = math.sqrt(2)*x*tmp2 - math.sqrt(k-1)*tmp1*ef
            y = ef * y / math.sqrt(k)
            tmp1 = tmp2
            tmp2 = y
    return y



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


"""Module that computes Diagonal of Gabor frame operator

Ported from ltfat_2.1.0/gabor/gabframediag.m

.. moduleauthor:: Denis Arrivault
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.sigproc.fir2long import fir2long
from ltfatpy.gabor.dgtlength import dgtlength


def gabframediag(g, a, M, L):
    """Diagonal of Gabor frame operator

    - Usage:

        | ``d = gabframediag(g, a, M, L)``

    - Input parameters:

    :param numpy.ndarray g: Window function
    :param int a: Length of time shift
    :param int M: Number of channels
    :param int L: Length of transform to do

    - Output parameters:

    :returns: Diagonal stored as a column vector
    :rtype: numpy.ndarray

    ``gabframediag(g, a, M, L)`` computes the diagonal of the Gabor frame
    operator with respect to the window **g** and parameters **a** and **M**.
    The diagonal is stored a as column vector of length **L**.

    The diagonal of the frame operator can for instance be used as a
    preconditioner.

    .. seealso:: :func:`~ltfatpy.gabor.dgt.dgt`
    """
    # Verify a, M and get L
    Luser = dgtlength(L, a, M)
    if L != Luser:
        raise ValueError(("Incorrect transform length L={0:d} specified." +
                          " Next valid length  is L={0:d}. See the help" +
                          " of DGTLENGTH for the requirements.").format(L,
                         Luser))

    # Determine the window
    (g, info) = _call_gabwin(g, a, M, L)
    if L < info['gl']:
        raise ValueError('Window is too long.')

    # compute the diagonal
    glong2 = np.abs(fir2long(g, L))**2
    N = L//a

    # The diagonal is a-periodic, so compute a single period by summing up
    # glong2 in slices.
    d = np.tile(np.sum(glong2.reshape(N, a), axis=0), (N,)) * M
    return d


def _call_gabwin(g, a, M, L):
    """ Internal use only """

    from ltfatpy.gabor.gabwin import gabwin
    return gabwin(g, a, M, L)



# -*- coding: utf-8 -*-
# ######### COPYRIGHT #########
# Credits
# #######
#
# Copyright(c) 2015-2025
# ----------------------
#
# * `LabEx Archimède <http://labex-archimede.univ-amu.fr/>`_
# * `Laboratoire d'Informatique Fondamentale <http://www.lif.univ-mrs.fr/>`_
#   (now `Laboratoire d'Informatique et Systèmes <http://www.lis-lab.fr/>`_)
# * `Institut de Mathématiques de Marseille <http://www.i2m.univ-amu.fr/>`_
# * `Université d'Aix-Marseille <http://www.univ-amu.fr/>`_
#
# This software is a port from LTFAT 2.1.0 :
# Copyright (C) 2005-2025 Peter L. Soendergaard <peter@sonderport.dk>.
#
# Contributors
# ------------
#
# * Denis Arrivault <contact.dev_AT_lis-lab.fr>
# * Florent Jaillet <contact.dev_AT_lis-lab.fr>
#
# Description
# -----------
#
# ltfatpy is a partial Python port of the
# `Large Time/Frequency Analysis Toolbox <http://ltfat.sourceforge.net/>`_,
# a MATLAB®/Octave toolbox for working with time-frequency analysis and
# synthesis.
#
# Version
# -------
#
# * ltfatpy version = 1.1.2
# * LTFAT version = 2.1.0
#
# Licence
# -------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ######### COPYRIGHT #########


""" Module of group thresholding

Ported from ltfat_2.1.0/sigproc/groupthresh.m

.. moduleauthor:: Florent Jaillet
"""

from __future__ import print_function, division

import numpy as np

from ltfatpy.comp.assert_sigreshape_pre import assert_sigreshape_pre
from ltfatpy.comp.assert_sigreshape_post import assert_sigreshape_post
from ltfatpy.sigproc.thresh import thresh


def groupthresh(xi, lamb, dim=1, group_type='group', thresh_type='soft'):
    """Group thresholding

    - Usage:

        | ``xo = groupthresh(xi, lamb)``
        | ``xo = groupthresh(xi, lamb, dim)``
        | ``xo = groupthresh(xi, lamb, ...)``

    - Input parameters:

    :param numpy.ndarray xi: Input array. ``xi`` must be a two-dimensional
        array, with dimension ``0`` labelling groups, and dimension ``1``
        labelling members. This means that the groups are the row vectors of
        the input (the vectors along the dimension ``1``).
    :param float lamb: Threshold
    :param int dim: Dimension along which to choose the groups
        (default ``dim=1``)
    :param str group_type: Optional flag specifying the grouping behaviour
        (see possible values below)
    :param str thresh_type: Optional flag specifying the type of thresholding
        within each group (see the parameter ``thresh_type`` in the help
        of :func:`~ltfatpy.sigproc.thresh.thresh` for possible values,
        default ``thresh_type=soft``)

    - Output parameters:

    :returns: Array of the same shape as **xi** containing data from **xi**
        after group thresholding
    :rtype: numpy.ndarray

    ``groupthresh(xi, lamb)`` performs group thresholding on ``xi``, with
    threshold ``lamb``.

    ``groupthresh(xi, lamb, dim)`` chooses groups along dimension ``dim``.

    The parameter **group_type** can take the following values:
        =========== =======================================================
        ``'group'`` Shrinks all coefficients within a given group according
                    to the value of the :math:`l^2` norm of the group in
                    comparison to the threshold ``lamb``.
                    This is the default

        ``'elite'`` Shrinks all coefficients within a given group according
                    to the value of the :math:`l^1` norm of the group in
                    comparison to the threshold value ``lamb``
        =========== =======================================================

    .. seealso::
        :func:`~ltfatpy.sigproc.thresh.thresh`

    - References:
        :cite:`Kowalski08sparsity,kowalski2009mixed,yu2008audio`
    """

    # Note: This function doesn't support the handling of sparse matrices
    # available in the Octave version. Only full numpy arrays are supported in
    # input and output.

    if not isinstance(lamb, float):
        raise TypeError('lamb must be a float')

    # dim (the time or frequency selector) is handled by assert_sigreshape_pre
    xi, L, NbMembers, NbGroups, dim, permutedshape, order = \
        assert_sigreshape_pre(xi, None, dim)

    # Dense case (this Python port doesn't handle the sparse matrix case)
    xo = np.zeros(xi.shape, dtype=xi.dtype)

    if group_type == 'group':
        groupnorm = np.sqrt(np.sum(np.abs(xi)**2., axis=0))
        w = thresh(groupnorm, lamb, thresh_type=thresh_type)[0] / groupnorm

        # Clean w for NaN. NaN appears if the input has a group with norm
        # exactly 0.
        w[np.isnan(w)] = 0

        xo = xi * w

    elif group_type == 'elite':
        for ii in range(NbGroups):
            y = np.sort(np.abs(xi[:, ii]))[::-1]
            rhs = np.cumsum(y)
            rhs = rhs * lamb / (1. + lamb*np.arange(1., NbMembers+1))
            M_ii = np.nonzero(np.diff(np.sign(y-rhs)))[0]
            # Note: the test on M_ii in the Octave version of this function
            # is surprisingly written as find can only return non-zero values
            # or an empty array. Here an equivalent test is written using a
            # more explicit formulation.
            if M_ii.size != 0:
                tau_ii = float(lamb * np.linalg.norm(y[:M_ii[0]+1], 1) /
                               (1. + lamb*(M_ii[0]+1)))
            else:
                tau_ii = 0.

            xo[:, ii] = thresh(xi[:, ii], tau_ii, thresh_type=thresh_type)[0]

    xo = assert_sigreshape_post(xo, dim, permutedshape, order)

    return xo
